using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using AMF;

namespace ScalesOfAnarcrothe
{
    public partial class Form1 : Form
    {
        GexFile m_xSR1File;

        #region User Interface

        public Form1()
        {
            m_xSR1File = null;
            InitializeComponent();
            listView1.ItemActivate += new EventHandler(listView1_ItemActivate);
            listView1.AfterLabelEdit += new LabelEditEventHandler(listView1_AfterLabelEdit);
            listView2.ItemActivate += new EventHandler(listView2_ItemActivate);
            listView2.KeyPress += new KeyPressEventHandler(listView2_KeyPress);
            listView2.MouseUp += new MouseEventHandler(listView2_MouseUp);
            comboBox1.SelectedValueChanged += new EventHandler(comboBox1_SelectedValueChanged);
            comboBox1.Leave += new EventHandler(comboBox1_Leave);
            comboBox1.KeyPress += new KeyPressEventHandler(comboBox1_KeyPress);
        }

        void listView1_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            if (e.Label == null || e.Label == null)
            {
                e.CancelEdit = true;
                return;
            }

            if (e.Label.Length > 8)
            {
                e.CancelEdit = true;
                MessageBox.Show(
                    "Invalid object type",
                    "Object type must contain 8 or less characters.",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information
                );
            }

            for (int i = 0; i < listView2.Items.Count; i++)
            {
                if (listView2.Items[i].Text == listView1.SelectedItems[0].Text)
                {
                    listView2.Items[i].Text = e.Label;
                }
            }
        }

        void listView2_KeyPress(object sender, KeyPressEventArgs e)
        { 
            switch (e.KeyChar)
            {
                case (char)(int)Keys.Enter:
                {
                    ListViewItem lvItem = listView2.SelectedItems[0];
                    ShowComboBox(ref lvItem);
                    break;
                }
            }
        }

        void listView2_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                ListViewItem lvItem = listView2.GetItemAt(e.X, e.Y);
                ShowComboBox(ref lvItem);
            }
        }

        void comboBox1_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Verify that the user presses ESC.
            switch (e.KeyChar)
            {
                case (char)(int)Keys.Escape:
                {
                    // Reset the original text value, and then hide the ComboBox.
                    comboBox1.Text = listView2.SelectedItems[0].Text;
                    comboBox1.Visible = false;
                    break;
                }
                case (char)(int)Keys.Enter:
                {
                    // Hide the ComboBox.
                    comboBox1.Visible = false;
                    break;
                }
            }
        }

        void comboBox1_Leave(object sender, EventArgs e)
        {
            listView2.SelectedItems[0].Text = comboBox1.Text;
            comboBox1.Visible = false;
        }

        void comboBox1_SelectedValueChanged(object sender, EventArgs e)
        {
            listView2.SelectedItems[0].Text = comboBox1.Text;
            comboBox1.Visible = false;
        }

        void listView1_ItemActivate(object sender, EventArgs e)
        {
            listView1.SelectedItems[0].BeginEdit();
        }

        void listView2_ItemActivate(object sender, EventArgs e)
        {
            // listView2.SelectedItems[0].BeginEdit();
        }

        void ShowComboBox(ref ListViewItem lvItem)
        {
            // Make sure that an item is clicked.
            if (lvItem != null)
            {
                // If the item is disabled then don't show the combobox
                if (lvItem.ForeColor == Color.Gray)
                {
                    return;
                }

                // Get the bounds of the item that is clicked.
                Rectangle ItemRect = lvItem.Bounds;

                // Verify that the column is completely scrolled off to the left.
                if ((ItemRect.Left + listView2.Columns[0].Width) < 0)
                {
                    // If the cell is out of view to the left, do nothing.
                    return;
                }

                 // Verify that the column is partially scrolled off to the left.
                else if (ItemRect.Left < 0)
                {
                    // Determine if column extends beyond right side of ListView.
                    if ((ItemRect.Left + listView2.Columns[0].Width) > listView2.Width)
                    {
                        // Set width of column to match width of ListView.
                        ItemRect.Width = listView2.Width;
                        ItemRect.X = 0;
                    }
                    else
                    {
                        // Right side of cell is in view.
                        ItemRect.Width = listView2.Columns[0].Width + ItemRect.Left;
                        ItemRect.X = 2;
                    }
                }
                else if (listView2.Columns[0].Width > listView2.Width)
                {
                    ItemRect.Width = listView2.Width;
                }
                else
                {
                    ItemRect.Width = listView2.Columns[0].Width;
                    ItemRect.X = 2;
                }

                // Adjust the top to account for the location of the ListView.
                ItemRect.Y += listView2.Top;
                ItemRect.X += listView2.Left;

                // Assign calculated bounds to the ComboBox.
                comboBox1.Bounds = ItemRect;

                // Fill in the combo box.
                comboBox1.Items.Clear();
                for (int i = 0; i < listView1.Items.Count; i++)
                {
                    comboBox1.Items.Add(listView1.Items[i].Text);
                }

                // Set default text for ComboBox to match the item that is clicked.
                comboBox1.Text = lvItem.Text;

                // Display the ComboBox, and make sure that it is on top with focus.
                comboBox1.Visible = true;
                // comboBox1.DroppedDown = true;
                comboBox1.BringToFront();
                comboBox1.Focus();
            }
        }

        #endregion

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            comboBox1.Visible = false;

            OpenFileDialog OpenDlg = new OpenFileDialog();
            OpenDlg.CheckFileExists = true;
            OpenDlg.CheckPathExists = true;
            OpenDlg.Filter =    "Soul Reaver Mesh Files|*.SRObj;*.drm;*.pcm|" +
                                "Soul Reaver DRM Files (*.drm)|*.drm|" +
                                "Soul Reaver PCM Files (*.pcm)|*.pcm";
            OpenDlg.DefaultExt = "pcm";
            OpenDlg.FilterIndex = 0;
            if (OpenDlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    listView1.Items.Clear();
                    listView2.Items.Clear();
                    m_xSR1File = new GexFile(OpenDlg.FileName);
                    if (m_xSR1File.isObject)
                    {
                        m_xSR1File = null;
                        throw new Exception("Not an area file");
                    }

                    ListViewItem[] instanceTypes = new ListViewItem[m_xSR1File.instanceTypes.Length];
                    for (int i = 0; i < m_xSR1File.instanceTypes.Length; i++)
                    {
                        String[] item = { m_xSR1File.instanceTypes[i] };
                        instanceTypes[i] = new ListViewItem(item);
                        instanceTypes[i].Tag = i;
                    }
                    listView1.Items.AddRange(instanceTypes);

                    ListViewItem[] instances = new ListViewItem[m_xSR1File.instanceNames.Length];
                    for (int i = 0; i < m_xSR1File.instanceNames.Length; i++)
                    {
                        String[] item = { m_xSR1File.instanceNames[i] };
                        instances[i] = new ListViewItem(item);
                        instances[i].Tag = i;
                        if (listView1.FindItemWithText(item[0]) == null)
                        {
                            instances[i].ForeColor = Color.Gray;
                        }
                    }
                    listView2.Items.AddRange(instances);
                }
                catch (Exception error)
                {
                    MessageBox.Show(
                        error.Message,
                        "Could not load the file",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information
                    );
                }
            }
            return;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            comboBox1.Visible = false;

            try
            {
                if (m_xSR1File == null)
                {
                    throw new Exception("No file is loaded");
                }
                for (int i = 0; i < listView1.Items.Count; i++)
                {
                    m_xSR1File.instanceTypes[(int)listView1.Items[i].Tag] =
                        listView1.Items[i].Text;
                }
                for (int i = 0; i < listView2.Items.Count; i++)
                {
                    m_xSR1File.instanceNames[(int)listView2.Items[i].Tag] =
                        listView2.Items[i].Text;
                }
                m_xSR1File.Save();
            }
            catch (Exception error)
            {
                MessageBox.Show(
                    error.Message,
                    "Could not save the file",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information
                );
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            comboBox1.Visible = false;

            try
            {
                if (m_xSR1File == null)
                {
                    throw new Exception("No file is loaded");
                }

                SaveFileDialog SaveDlg = new SaveFileDialog();
                SaveDlg.Filter =    "Soul Reaver Mesh Files|*.SRObj;*.drm;*.pcm|" +
                                    "Soul Reaver DRM Files (*.drm)|*.drm|" +
                                    "Soul Reaver PCM Files (*.pcm)|*.pcm";
                SaveDlg.OverwritePrompt = true;
                try
                {
                    SaveDlg.DefaultExt = Path.GetExtension(m_xSR1File.fileName).ToLower();
                    switch (SaveDlg.DefaultExt)
                    {
                        case "srobj":
                            SaveDlg.FilterIndex = 1;
                            break;
                        case "drm":
                            SaveDlg.FilterIndex = 2;
                            break;
                    }
                }
                catch (Exception)
                {
                    SaveDlg.DefaultExt = "pcm";
                    SaveDlg.FilterIndex = 3;
                }
                SaveDlg.FileName = m_xSR1File.fileName;
                if (SaveDlg.ShowDialog() == DialogResult.OK)
                {
                    if (m_xSR1File == null)
                    {
                        throw new Exception("No file is loaded");
                    }
                    for (int i = 0; i < listView1.Items.Count; i++)
                    {
                        m_xSR1File.instanceTypes[(int)listView1.Items[i].Tag] =
                            listView1.Items[i].Text;
                    }
                    for (int i = 0; i < listView2.Items.Count; i++)
                    {
                        m_xSR1File.instanceNames[(int)listView2.Items[i].Tag] =
                            listView2.Items[i].Text;
                    }
                    m_xSR1File.Save(SaveDlg.FileName);
                }
            }
            catch (Exception error)
            {
                MessageBox.Show(
                    error.Message,
                    "Could not save the file",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information
                );
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}