using System;
using System.IO;
using System.Collections.Generic;

namespace AMF
{
    public class GexFile
    {
        public Byte[]           fileData;
        public String           fileName;
        public String           modelName;
        public UInt32           instanceCount;
        public UInt32           instanceStart;
        public String[]         instanceNames;
        public UInt32           instanceTypesStart;
        public String[]         instanceTypes;
        public UInt32           dataStart;
        public UInt32           modelData;
        public UInt32           vertexCount;
        public UInt32           vertexStart;
        public UInt32           polygonCount;
        public UInt32           polygonStart;
        public UInt32           boneCount;
        public UInt32           boneStart;
        public UInt32           materialStart;
        public UInt16           materialCount;
        public UInt32           bspTreeCount;
        public UInt32           bspTreeStart;
        public UInt32           indexCount { get { return 3 * polygonCount; } }
        public Boolean          isObject;
        public Boolean          isArea
        {
            get { return !isObject; }
            set { isObject = !value; }
        }

        public GexFile(string fileName)
        {
            this.fileName = fileName;

            FileStream file = new FileStream(fileName, FileMode.Open, FileAccess.Read);
            BinaryReader reader = new BinaryReader(file);

            fileData = reader.ReadBytes((int)file.Length);
            reader.BaseStream.Position = 0;

            // Get start of usefull data
            dataStart = ((reader.ReadUInt32() >> 9) << 11) + 0x00000800;

            // Get the type of model
            /*file.Position = dataStart + 0x4C;
            ulong typeTest1 = reader.ReadUInt64();
            file.Position = dataStart + 0x58;
            ulong typeTest2 = reader.ReadUInt64();

            file.Position = dataStart + 0x6C;
            ulong typeTester = 0xFFFFFFFFFFFFFFFF;
            if (reader.ReadUInt32() == 0xFFFF63C0)
            {
                typeTester = 0x0100020000000000;
            }*/

            file.Position = dataStart + 0x9C;
            ulong typeTest3 = reader.ReadUInt64();

            if (typeTest3 == 0xFFFFFFFFFFFFFFFF ||
                typeTest3 == 0x0100020000000000)
            {
                isArea = true;
                MapAreaFile(reader);
            }
            else
            {
                isObject = true;
                MapObjectFile(reader);
            }

            // Check for errors
            bool bError =
                (dataStart > reader.BaseStream.Length) ||
                (modelData > reader.BaseStream.Length) ||
                (vertexStart > reader.BaseStream.Length) ||
                (polygonStart > reader.BaseStream.Length) ||
                (boneCount > reader.BaseStream.Length) ||
                (materialStart > reader.BaseStream.Length);

            // Close the file
            reader.Close();
            file.Close();
            reader = null;
            file = null;

            if (bError)
            {
                throw new Exception("Wrong file type or not a mesh file");
            }
        }

        private void MapObjectFile(BinaryReader reader)
        {
            reader.BaseStream.Position = dataStart + 0x00000024;
            reader.BaseStream.Position = dataStart + reader.ReadUInt32();
            modelName = new String(reader.ReadChars(8)).TrimEnd('\0');
            reader.BaseStream.Position = dataStart + 0x0000000C;
            reader.BaseStream.Position = dataStart + reader.ReadUInt32();
            modelData = dataStart + reader.ReadUInt32();
            reader.BaseStream.Position = modelData;
            vertexCount = reader.ReadUInt32();
            vertexStart = dataStart + reader.ReadUInt32();
            reader.BaseStream.Position += 0x00000008;
            polygonCount = reader.ReadUInt32();
            polygonStart = dataStart + reader.ReadUInt32();
            boneCount = reader.ReadUInt32();
            boneStart = dataStart + reader.ReadUInt32();
            materialCount = 0;

            return;
        }

        private void MapAreaFile(BinaryReader reader)
        {
            reader.BaseStream.Position = dataStart + 0x78;
            instanceCount = reader.ReadUInt32();
            instanceStart = dataStart + reader.ReadUInt32();
            instanceNames = new String[instanceCount];
            for (int i = 0; i < instanceCount; i++)
            {
                reader.BaseStream.Position = instanceStart + 0x4C * i;
                instanceNames[i] = new String(reader.ReadChars(8)).TrimEnd('\0');
            }
            reader.BaseStream.Position = dataStart + 0x8C;
            instanceTypesStart = dataStart + reader.ReadUInt32();
            reader.BaseStream.Position = instanceTypesStart;
            List<String> instanceList = new List<String>();
            while (reader.ReadByte() != 0xFF)
            {
                reader.BaseStream.Position--;
                instanceList.Add(new String(reader.ReadChars(8)).TrimEnd('\0'));
                reader.BaseStream.Position += 0x08;
            }
            instanceTypes = instanceList.ToArray();
            reader.BaseStream.Position = dataStart + 0x98;
            reader.BaseStream.Position = dataStart + reader.ReadUInt32();
            modelName = new String(reader.ReadChars(8)).TrimEnd('\0');
            reader.BaseStream.Position = dataStart;
            modelData = dataStart + reader.ReadUInt32();
            reader.BaseStream.Position = modelData + 0x10;
            vertexCount = reader.ReadUInt32();
            polygonCount = reader.ReadUInt32();
            reader.BaseStream.Position += 0x04;
            vertexStart = dataStart + reader.ReadUInt32();
            polygonStart = dataStart + reader.ReadUInt32();
            boneCount = 0;
            boneStart = 0;
            reader.BaseStream.Position += 0x10;
            materialStart = dataStart + reader.ReadUInt32();
            materialCount = 0;
            reader.BaseStream.Position += 0x0C;
            bspTreeCount = reader.ReadUInt32();
            bspTreeStart = dataStart + reader.ReadUInt32();
            return;
        }

        public void Save()
        {
            Save(fileName);
        }

        public void Save(String fileName)
        {
            FileStream file = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.Write);
            BinaryWriter writer = new BinaryWriter(file);

            writer.Write(fileData);

            for (int i = 0; i < instanceCount; i++)
            {
                writer.BaseStream.Position = instanceStart + 0x4C * i;
                writer.Write(instanceNames[i].ToCharArray());
            }

            for (int i = 0; i < instanceTypes.Length; i++)
            {
                writer.BaseStream.Position = instanceTypesStart + 0x10 * i;
                writer.Write(instanceTypes[i].ToCharArray());
            }

            writer.Close();
            file.Close();
            writer = null;
            file = null;

            this.fileName = fileName;
        }
    }
}