using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ScalesOfAnarcrothe
{
    public partial class ListViewEx : ListView
    {
        private const int WM_HSCROLL = 0x114;
        private const int WM_VSCROLL = 0x115;
        private const int WM_NOTIFY = 0x004E;
        private const int HDN_FIRST = (0 - 300);
        private const int HDN_BEGINTRACKA = (HDN_FIRST - 6);
        private const int HDN_BEGINTRACKW = (HDN_FIRST - 26);
        private const int WM_LBUTTONDBLCLK = 0x0203;
        private const int WM_SETCURSOR = 0x0020;
        private const int GWL_STYLE = -16;
        private const int GWL_EXSTYLE = -20;
        private const int LBS_DISABLENOSCROLL = 0x1000; // 0x1000L
        private const int WS_VSCROLL = 0x00200000;
        private const int WS_HSCROLL = 0x00100000;
        private bool locked = true;
        private HeaderControl hdrCtrl = null;

        /// <summary>
        /// Property to turn on and off the ability to size the column headers.
        /// </summary>
        public bool LockColumnSize
        {
            get { return locked; }
            set
            {
                locked = value;
            }
        }

        /// <summary>
        /// Notify message header structure.
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        private struct NMHDR
        {
            public IntPtr hwndFrom;
            public int idFrom;
            public int code;
        }

        /// <summary>
        /// Class used to capture window messages for the header of the list view
        /// control.  
        /// </summary>
        private class HeaderControl : NativeWindow
        {
            private ListViewEx parentListView = null;

            public HeaderControl(ListViewEx m)
            {
                parentListView = m;
                IntPtr header = SendMessage(m.Handle,
                    (0x1000 + 31), IntPtr.Zero, IntPtr.Zero);
                this.AssignHandle(header);
            }

            protected override void WndProc(ref Message message)
            {
                switch (message.Msg)
                {
                    case WM_LBUTTONDBLCLK:
                    case WM_SETCURSOR:
                    if (parentListView.LockColumnSize)
                    {
                        message.Result = (IntPtr)1;
                        return;
                    }
                    break;
                }
                
                base.WndProc(ref message);
            }
        }

        [DllImport("User32", CharSet = CharSet.Auto)]
        private static extern int GetWindowLong(IntPtr hWnd, int Index);

        [DllImport("User32", CharSet = CharSet.Auto)]
        private static extern int SetWindowLong(IntPtr hWnd, int Index, int Value);

        [DllImport("User32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);

        public ListViewEx()
        {
            InitializeComponent();

            //int style = GetWindowLong(this.Handle, GWL_STYLE);
            //int exStyle = GetWindowLong(this.Handle, GWL_EXSTYLE);

            //// Modify existing style values
            //style = style | WS_VSCROLL;

            //// Set new style values
            //SetWindowLong(this.Handle, GWL_STYLE, style);
            //SetWindowLong(this.Handle, GWL_EXSTYLE, exStyle);
        }

        protected override void WndProc(ref Message msg)
        {
            switch (msg.Msg)
            {
                case WM_VSCROLL:
                case WM_HSCROLL:
                {
                    // Move focus to the ListView to cause ComboBox to lose focus.
                    this.Focus();
                    break;
                }
                case WM_NOTIFY:
                {
                    NMHDR nmhdr = (NMHDR)msg.GetLParam(typeof(NMHDR));
                    switch (nmhdr.code)
                    {
                        case HDN_BEGINTRACKA:
                        case HDN_BEGINTRACKW:
                            if (locked)
                            {
                                msg.Result = (IntPtr)1;
                                return;
                            }
                            break;
                    }
                    break;
                }
            }

            base.WndProc(ref msg);
        }


        /// <summary>
        /// When the control is created capture the messages for the header. 
        /// </summary>
        protected override void OnCreateControl()
        {
            base.OnCreateControl();
            hdrCtrl = new HeaderControl(this);
        }

        /// <summary>
        /// Capture CTRL+ to prevent resize of all columns.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (e.KeyValue == 107 && e.Modifiers == Keys.Control && locked)
            {
                e.Handled = true;
            }
            else
            {
                base.OnKeyDown(e);
            }
        }
    }
}
